@@ -625,10 +625,12 @@ int check_deltas(struct ploop_disk_images_data *di, char **images,
 		int flags = CHECK_DETAILED | (di ? (CHECK_DROPINUSE | CHECK_REPAIR_SPARSE) : 0);
 		int raw_delta = (raw && i == 0);
 		__u32 cur_blocksize = raw_delta ? *blocksize : 0;
+		int ro = (images[i+1] != NULL);
 
-		ret = ploop_check(images[i], flags, 0, raw_delta, 0, &cur_blocksize);
+		ret = ploop_check(images[i], flags, ro, raw_delta, 0, &cur_blocksize);
 		if (ret) {
-			ploop_err(0, "%s : irrecoverable errors", images[i]);
+			ploop_err(0, "%s : irrecoverable errors (%s)",
+					images[i], ro ? "ro" : "rw");
 			break;
 		}
 		if (*blocksize == 0)
@@ -644,6 +646,48 @@ int check_deltas(struct ploop_disk_images_data *di, char **images,
 	return ret;
 }
 
+int check_dd(struct ploop_disk_images_data *di, const char *uuid)
+{
+	char **images;
+	__u32 blocksize;
+	int raw;
+	int ret;
+	char **devices;
+
+	if (ploop_lock_dd(di))
+		return SYSEXIT_LOCK;
+
+	ret = ploop_get_devs(di, &devices);
+	if (ret == 0) {
+		ploop_err(0, "Can't check, in use by device: %s", devices[0]);
+		ploop_free_array(devices);
+		return SYSEXIT_PLOOPINUSE;
+	}
+	else if (ret == -1)
+		/* Some system error */
+		return SYSEXIT_SYS;
+
+	images = make_images_list(di, (uuid) ? uuid : di->top_guid, 0);
+	if (!images) {
+		/* this might fail for a number of reasons, so
+		 * let's return something more or less generic.
+		 */
+		ret = SYSEXIT_DISKDESCR;
+		goto out;
+	}
+
+	blocksize = di->blocksize;
+	raw = (di->mode == PLOOP_RAW_MODE);
+
+	ret = check_deltas(di, images, raw, &blocksize);
+
+	free_images_list(images);
+out:
+	ploop_unlock_dd(di);
+
+	return ret;
+}
+
 int check_deltas_same(const char *img1, const char *img2) {
 	int i, ret;
 	int fd[2] = { -1, -1 };
