@@ -101,6 +101,35 @@ int find_delta_names(const char * device, int start_level, int end_level,
 	return 0;
 }
 
+int find_level_by_delta(const char *device, const char *delta)
+{
+	int i, top_level;
+	char path[PATH_MAX];
+	char nbuf[PATH_MAX];
+	char delta_r[PATH_MAX];
+
+	if (memcmp(device, "/dev/", 5) == 0)
+		device += 5;
+
+	if (realpath(delta, delta_r) == NULL) {
+		ploop_err(errno, "Can't resolve %s", delta);
+		return -1;
+	}
+	if (ploop_get_attr(device, "top", &top_level))
+		return -1;
+
+	for (i = 0; i <= top_level; i++) {
+		snprintf(path, sizeof(path), "/sys/block/%s/pdelta/%d/image",
+			 device, i);
+
+		if (read_line(path, nbuf, sizeof(nbuf)))
+			return -1;
+		if (strcmp(nbuf, delta_r) == 0)
+			return i;
+	}
+
+	return -1;
+}
 int ploop_get_attr(const char * device, const char * attr, int * res)
 {
 	char path[PATH_MAX];
@@ -156,7 +185,7 @@ int ploop_get_size(const char * device, off_t * res)
 		close(fd);
 		return SYSEXIT_DEVIOC;
 	}
-	*res >>= 9;
+	*res >>= PLOOP1_SECTOR_LOG;
 	close(fd);
 
 	return 0;
@@ -267,7 +296,7 @@ close_dir:
 	return -1;
 }
 
-/* Find device(s) by base delta and return name(s)
+/* Find device(s) by base ( & top ) delta and return name(s)
  * in a NULL-terminated array pointed to by 'out'.
  * Note that
  *  - if 0 is returned, 'out' should be free'd using
@@ -279,8 +308,8 @@ close_dir:
  *   0 found
  *   1 not found
  */
-int ploop_get_dev_by_delta(const char *component_name, const char *delta,
-		char **out[])
+int ploop_get_dev_by_delta(const char *delta, const char *topdelta,
+		const char *component_name, char **out[])
 {
 	char fname[PATH_MAX];
 	char delta_r[PATH_MAX];
@@ -332,6 +361,13 @@ int ploop_get_dev_by_delta(const char *component_name, const char *delta,
 		if (strcmp(image, delta_r))
 			continue;
 
+		if (topdelta != NULL) {
+			if (!(ploop_find_top_delta_name_and_format(
+					de->d_name, image, sizeof(image), NULL, 0) == 0 &&
+					strcmp(image, topdelta) == 0))
+				continue;
+		}
+
 		snprintf(fname, sizeof(fname), "/sys/block/%s/pstate/cookie",
 				de->d_name);
 		err = read_line_quiet(fname, cookie, sizeof(cookie));
@@ -397,13 +433,13 @@ void ploop_free_array(char *array[])
 	free(array);
 }
 
-int ploop_find_dev(const char *delta, const char *component_name,
+int ploop_find_dev(const char *component_name, const char *delta,
 		char *out, int size)
 {
 	int ret;
 	char **devs;
 
-	ret = ploop_get_dev_by_delta(delta,
+	ret = ploop_get_dev_by_delta(delta, NULL,
 			/* We only need one device, so
 			 * always set component_name */
 			component_name ? component_name : "",
