@@ -381,6 +381,7 @@ struct send_data {
 	int err_no;
 	pthread_mutex_t mutex;
 	pthread_cond_t cond;
+	pthread_cond_t cond_sent;
 	int has_data;
 };
 
@@ -388,9 +389,9 @@ static void *send_thread(void *data) {
 	struct send_data *sd = data;
 	int done;
 
+	pthread_mutex_lock(&sd->mutex);
 	do {
-		pthread_mutex_lock(&sd->mutex);
-		if (!sd->has_data) {
+		while (!sd->has_data) {
 			pthread_cond_wait(&sd->cond, &sd->mutex);
 		}
 		sd->ret = send_buf(sd->fd, sd->buf,
@@ -399,8 +400,9 @@ static void *send_thread(void *data) {
 			sd->err_no = errno;
 		done = (sd->len == 0 && sd->pos == 0);
 		sd->has_data = 0;
-		pthread_mutex_unlock(&sd->mutex);
+		pthread_cond_signal(&sd->cond_sent);
 	} while (!done);
+	pthread_mutex_unlock(&sd->mutex);
 
 	return NULL;
 }
@@ -408,6 +410,9 @@ static void *send_thread(void *data) {
 #define async_send(length, position)			\
 do {							\
 	pthread_mutex_lock(&sd.mutex);			\
+	while (sd.has_data) {				\
+		pthread_cond_wait(&sd.cond_sent, &sd.mutex);	\
+	}						\
 	if (sd.ret) {					\
 		ploop_err(sd.err_no, "write error");	\
 		ret = sd.ret;				\
@@ -417,8 +422,8 @@ do {							\
 	sd.len = (length);				\
 	sd.pos = (position);				\
 	sd.has_data = 1;				\
-	pthread_mutex_unlock(&sd.mutex);		\
 	pthread_cond_signal(&sd.cond);			\
+	pthread_mutex_unlock(&sd.mutex);		\
 } while (0)
 
 #define do_pread(length, position)			\
@@ -462,6 +467,7 @@ int ploop_copy_send(struct ploop_copy_send_param *arg)
 	struct send_data sd = {
 		.mutex = PTHREAD_MUTEX_INITIALIZER,
 		.cond = PTHREAD_COND_INITIALIZER,
+		.cond_sent = PTHREAD_COND_INITIALIZER,
 	};
 
 	if (!arg)
