@@ -1193,7 +1193,7 @@ static int reread_part(const char *device)
 		ploop_err(errno, "Can't open %s", device);
 		return -1;
 	}
-	if (ioctl(fd, BLKRRPART, 0) < 0)
+	if (do_ioctl(fd, BLKRRPART) < 0)
 		ploop_err(errno, "BLKRRPART %s", device);
 	close(fd);
 
@@ -1600,21 +1600,24 @@ err:
 }
 
 /* Checks a mount point hosting a ploop image
- * for bad (i.e. not recommended) mount options.
+ * for bad (i.e. not recommended) mount options
+ * and display a warning message if such an option
+ * is present.
  *
  * Returns:
  *  -1: internal error
  *   1: bad mount option found
  *   0: everything is fine
+ *
+ * TODO: change a warning into an error
  */
 static int check_host_ext4_mount_opts(const char *file)
 {
 	struct stat st;
 	char buf[PATH_MAX * 4];
 	FILE *fp;
-	int ret;
 	const char *bad_opt="data=writeback";
-	static int warned = 0;
+	int ret = -1;
 
 	if (stat(file, &st)) {
 		ploop_err(errno, "Can't stat %s", file);
@@ -1627,7 +1630,6 @@ static int check_host_ext4_mount_opts(const char *file)
 		return -1;
 	}
 
-	ret = -1;
 	while (fgets(buf, sizeof(buf), fp)) {
 		int n;
 		unsigned int major, minor;
@@ -1647,26 +1649,22 @@ static int check_host_ext4_mount_opts(const char *file)
 		opt = strrchr(buf, ' ');
 		if (opt == NULL) /* should never happen */
 			break;
+
+		ret = 0;
 		/* check mount options */
-		if (strstr(opt, bad_opt) == NULL) {
-			ret = 0;
-			break;
-		}
-		if (!warned) {
+		if (strstr(opt, bad_opt) != NULL) {
+			/* ret = 1; FIXME: warning for now */
 			ploop_log(-1, "WARNING: %s is mounted with %s "
 					"not recommended for ploop; "
 					"please use data=ordered instead",
 					target, bad_opt);
-			warned = 1;
 		}
-
-		ret = 0; /* FIXME: just a warning for now */
-		break;
+		goto out;
 	}
 
-	if (ret == -1)
-		ploop_log(0, "Warning: mount point not found for %s", file);
+	ploop_log(0, "Warning: mount point not found for %s", file);
 
+out:
 	fclose(fp);
 
 	return ret;
@@ -1679,29 +1677,31 @@ static int check_host_ext4_mount_opts(const char *file)
 #define EXT4_EXTENTS_FL		0x00080000 /* Inode uses extents */
 #endif
 
-static int check_mount_restrictions(struct ploop_mount_param *param, const char *fname)
+static int check_ext4_mount_restrictions(const char *fname)
 {
 	struct statfs st;
 	int fd, ret;
 	long flags;
 
-	ret = check_host_ext4_mount_opts(fname);
-	if (ret > 0) /* ignore internal errors */
-		return ret;
-
-	/* FIXME: */
-	if (getenv("PLOOP_SKIP_EXT4_EXTENTS_CHECK") != NULL)
-		return 0;
 	if (statfs(fname, &st) < 0) {
 		ploop_err(errno, "Unable to statfs %s", fname);
 		return -1;
 	}
-	if (st.f_type == EXT4_SUPER_MAGIC) {
+
+	if (st.f_type != EXT4_SUPER_MAGIC)
+		return 0;
+
+	ret = check_host_ext4_mount_opts(fname);
+	if (ret)
+		return ret;
+
+	if (getenv("PLOOP_SKIP_EXT4_EXTENTS_CHECK") == NULL) {
 		fd = open(fname, O_RDONLY);
 		if (fd < 0) {
 			ploop_err(errno, "Can't open %s", fname);
 			return -1;
 		}
+
 		if (ioctl(fd, FS_IOC_GETFLAGS, &flags) < 0) {
 			ploop_err(errno, "FS_IOC_GETFLAGS %s", fname);
 			close(fd);
@@ -1719,6 +1719,30 @@ static int check_mount_restrictions(struct ploop_mount_param *param, const char
 	return 0;
 }
 
+static int check_mount_restrictions(char **images)
+{
+	int i, ret;
+	struct stat st;
+	dev_t prev_dev = 0;
+
+	for (i = 0; images[i] != NULL; i++) {
+		if (stat(images[i], &st) < 0) {
+			ploop_err(errno, "Unable to statfs %s", images[i]);
+			return -1;
+		}
+		/* device already checked */
+		if (st.st_dev == prev_dev)
+			continue;
+
+		ret = check_ext4_mount_restrictions(images[i]);
+		if (ret)
+			return ret;
+
+		prev_dev = st.st_dev;
+	}
+	return 0;
+}
+
 int ploop_mount(struct ploop_disk_images_data *di, char **images,
 		struct ploop_mount_param *param, int raw)
 {
@@ -1755,7 +1779,7 @@ int ploop_mount(struct ploop_disk_images_data *di, char **images,
 	} else if (di)
 		blocksize = di->blocksize;
 
-	if (check_mount_restrictions(param, images[0]))
+	if (check_mount_restrictions(images))
 		return SYSEXIT_MOUNT;
 
 	if (di && (ret = check_and_restore_fmt_version(di)))
