@@ -279,8 +279,9 @@ int relocmap2relocblks(struct relocmap *relocmap, int lvl, __u32 a_h, __u32 n_sc
 			struct ploop_relocblks_ctl **relocblks_pp);
 PL_EXT int ploop_check(char *img, int flags, int ro, int raw, int verbose,
 		__u32 *blocksize_p);
-PL_EXT int check_deltas(struct ploop_disk_images_data *di, char **images,
+int check_deltas(struct ploop_disk_images_data *di, char **images,
 		int raw, __u32 *blocksize);
+PL_EXT int check_dd(struct ploop_disk_images_data *di, const char *uuid);
 PL_EXT int check_deltas_same(const char *img1, const char *img2);
 /* Logging */
 #define LOG_BUF_SIZE	8192
