@@ -77,7 +77,9 @@ int plooptool_merge(int argc, char ** argv)
 			}
 			break;
 		case 'u':
-			param.guid = optarg;
+			param.guid = parse_uuid(optarg);
+			if (!param.guid)
+				return SYSEXIT_PARAM;
 			break;
 		case 'A':
 			param.merge_all = 1;
@@ -100,11 +102,11 @@ int plooptool_merge(int argc, char ** argv)
 			return SYSEXIT_PARAM;
 		}
 
-		ret = read_dd(&di, argv[0]);
+		ret = ploop_open_dd(&di, argv[0]);
 		if (ret)
 			return ret;
 		ret = ploop_merge_snapshot(di, &param);
-		ploop_free_diskdescriptor(di);
+		ploop_close_dd(di);
 	} else {
 		if (device == NULL) {
 			if (argc < 2) {
